######################################################################
# GZIP.pm - This is PyukiWiki, yet another Wiki clone.
# $Id: GZIP.pm,v 1.297 2012/09/11 00:43:00 papu Exp $
# Build on 2012-09-11 08:19:35
#
# "Nana::GZIP" ver 0.3 $$
# Author: Nanami
# http://nanakochi.daiba.cx/
# Copyright (C) 2004-2007 Nekyo
# Copyright (C) 2005-2012 PyukiWiki Developers Team
# http://pyukiwiki.info/
# Based on YukiWiki http://www.hyuki.com/yukiwiki/
# Powerd by PukiWiki http://pukiwiki.sfjp.jp/
# License: GPL3 and/or Artistic or each later version
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Return:CRLF Code=Shift-JIS 1TAB=4Spaces
######################################################################
#
# use Nana::GZIP;
# my $gz=new Nana::GZIP(
#            prog=>"gzip or pigz or zlib or other",
#            level=>"fast or best or other",
#            path=>(gzip path optional)
#        );
# $compressdata=$gz->compress($data);
# $extractdata=$gz->uncompress($data);
#
# see
# http://suika.fam.cx/~wakaba/wiki/sw/n/Perl%E3%81%A7%E3%81%AEgzip%E3%81%AE%E5%9C%A7%E7%B8%AE%E3%83%BB%E5%B1%95%E9%96%8B
# http://www.submit.ne.jp/1500
# http://d.hatena.ne.jp/yukiex/20080227/
######################################################################

#use Compress::Zlib;
package Nana::GZIP;
$VERSION="0.3";
use strict;
use Nana::ServerInfo;

$GZIP::INIT=0;
$GZIP::FORCEPIGZ=0;
$GZIP::FORCEGZIP=0;

######################################################################

$GZIP::INITED=0;
$GZIP::PATH;
$GZIP::FORCE;
$GZIP::FAST;
$GZIP::BEST;

$GZIP::BEFOREPROG="GZIP.pm test";
$GZIP::BEFORELEVE;
$GZIP::BEFORCOMPRESSFLAG;
$GZIP::BEFOREUNCOMPRESSFLAG;
$GZIP::BEFOREPATH;


# multicore gzip tips: http://www.submit.ne.jp/1500			# comment


# init

# $gz=new Nana::GZIP(prog=>"gzip or pigz", level="fast or best or default or 1..9");

sub new {
	my($class,%hash)=@_;
	my $ret;
$::debug.="GZIP.pm: init\n";
	if($hash{prog} ne $GZIP::BEFOREPROG || $hash{level} ne $GZIP::BEFORELEVEL) {
		my $pigz_command='pigz1';
		my $gzip_command='gzip1';
		my $zlib_command='zlib';

		my $prog=lc $hash{prog};
		my $path=lc $hash{prog};

		if($prog!~/^($gzip_command|$pigz_command|$zlib_command|nouse|)$/) {
			return bless {
				prog=>"",
				level=>"",
				path=>"",
				init=>0,
			}, $class;
		}

		# MpĎs
		$path=(split(/ /, $path))[0];
		if($-x $path) {
			$::debug.="GZIP.pm: Force $path\n";
			return bless {
				prog=>"custom",
				path=>$hash{path},
				compressflag=>$hash{compressflag},
				uncompressflag=>$hash{uncompressflag},
				init=>1,
			}, $class;
		}

		# 
		my $_execpath="/usr/local/bin:/usr/bin:/bin:$ENV{PATH}";
		my $_force="--force";
		my $_fast="--fast";
		my $_best="--best";
		my $_decompress="--decompress";
		my ($path, $pathbak);
		my ($forceflag, $fastflag, $bestflag, $decompressflag);
		my $info=new Nana::ServerInfo;

		if($prog eq "" || $prog eq $gzip_command) {
			my $found=0;
			foreach(split(/:/,$_execpath)) {
				if(-x "$_/$gzip_command") {
					$path="$_/$gzip_command" ;
#					$prog=$gzip_command;
					if(open(PIPE,"$path --help 2>&1|")) {
						foreach(<PIPE>) {
							$forceflag="$_force" if(/$_force/);
							$fastflag="$_fast" if(/$_fast/);
							$bestflag="$_best" if(/$_best/);
							$decompressflag=$_decompress if(/$_decompress/);
						}
						$found=1;
						close(PIPE);
					}
				}
			}
			$path="" if($found eq 0);
		}
		$pathbak=$path;
		if($prog eq "" && $info->core > 3 || $prog eq $pigz_command) {
			my $found=0;
			foreach(split(/:/,$_execpath)) {
				if(-x "$_/$pigz_command") {
					$path="$_/$pigz_command" ;
#					$prog=$gzip_command;
					if(open(PIPE,"$path --help 2>&1|")) {
						foreach(<PIPE>) {
							$forceflag="$_force" if(/$_force/);
							$fastflag="$_fast" if(/$_fast/);
							$bestflag="$_best" if(/$_best/);
							$decompressflag=$_decompress if(/$_decompress/);
						}
						$found=1;
						close(PIPE);
					}
				}
			}
			$path=$pathbak if($found eq 0);
		}
		$pathbak=$path;
		if($path eq "" || $prog eq $zlib_command) {
			if(&load_module("Compress::Zlib")) {
				$path="zlib";
			}
		}

		my $level=
			lc $hash{level} eq "best" ? " $forceflag $bestflag " :
			lc $hash{level} eq "fast" ? " $forceflag $fastflag " :
			" $forceflag $fastflag ";

		$decompressflag="$forceflag $decompressflag";
		$GZIP::BEFOREPROG=$hash{prog};
		$GZIP::BEFORELEVEL=$hash{level};
		$GZIP::BEFORCOMPRESSFLAG=$level;
		$GZIP::BEFOREUNCOMPRESSFLAG=$decompressflag;
		$GZIP::BEFOREPATH=$path;
		$::debug.="GZIP.pm: auto detect gzip (compress) : $GZIP::BEFOREPATH $GZIP::BEFORCOMPRESSFLAG\nGZIP.pm: auto detect gzip (uncompress) : $GZIP::BEFOREPATH $GZIP::BEFOREUNCOMPRESSFLAG\n";
		return bless {
			prog=>$hash{prog},
			level=>$hash{level},
			path=>$hash{path},
			compressflag=>$level,
			uncompressflag=>$decompressflag,
			_path=>$path,
			init=>1,
		}, $class;
	}
	return bless {
		prog=>$GZIP::BEFOREPROG,
		level=>$GZIP::BEFORELEVEL,
		path=>$hash{path},
		compressflag=>$GZIP::BEFORCOMPRESSFLAG,
		uncompressflag=>$GZIP::BEFOREUNCOMPRESSFLAG,
		_path=>$GZIP::BEFOREPATH,
		init=>1
	}, $class;
}

sub fifo {
	my ($cmd, $post)=@_;

	my $path=&mktemp($ENV{REMOTE_HOST} . $ENV{HTTP_USER_AGENT});
	my $mode = 0600;

	unlink($path);

	use POSIX;
	if(! mkfifo($path, $mode) ) {
		die "can not mkfifo($path, $mode): $!";
	}

	my $pid = fork();
	if ($pid ==0) {
		open(FIFO, "| $cmd > $path")
			or die "can not open $path: $!";
		print FIFO $post;
		close(FIFO);
		exit;
	}

	open(FIFO, "< ${path}")
		or die "can not open ${path}: $!";
	my $get;
	while(<FIFO>) {
		$get.=$_;
	}

	wait;
	close(FIFO);

	return $get;
}

sub mktemp {
	my($seed)=shift;

	return "$::cache_dir/fifotemp.test";
}

sub init {
	my ($self, $data)=@_;
 	return $self->{init};
}

sub compress {
	my ($self, $data)=@_;
	$::debug.="GZIP.pm compress cmd:$self->{_path}\n";
	if($self->{init}) {
		if($self->{_path} ne "zlib") {
			my $cmd=$self->{_path} . " " . $self->{compressflag};
			my $gziped=&fifo("$cmd", $data);
			return $gziped;
		}
		$::debug.="zlib writed\n";
		return Compress::Zlib::memGzip($data);
	}
	return $data;
}

sub uncompress {
	my ($self, $input)=@_;
	$::debug.="GZIP.pm extract cmd:$self->{_path}\n";
	if($self->{init}) {
		if($self->{_path} ne "zlib" ) {
			my $cmd=$self->{_path} . " " . $self->{uncompressflag};
			my $output=&fifo("$cmd", $input);
			return $output;
		}
		## Taken from Namazu <http://www.namazu.org/>, filter/gzip.pl
		my ($data)=shift;
		my ($s)=$input;
		my $flags = unpack('C', substr($s, 3, 1));
		$s = substr($s, 10);
		$s = substr($s, 2)  if ($flags & 0x04);
		$s =~ s/^[^\0]*\0// if ($flags & 0x08);
		$s =~ s/^[^\0]*\0// if ($flags & 0x10);
		$s = substr($s, 2)  if ($flags & 0x02);

		my $zl = Compress::Zlib::inflateInit
			(-WindowBits => - Compress::Zlib::MAX_WBITS());
		my ($inf, $stat) = $zl->inflate ($s);
		if ($stat == Compress::Zlib::Z_OK()
			|| $stat == Compress::Zlib::Z_STREAM_END()) {
			return $inf;
		} else {
			return 'Bad compressed data';
		}
	}
	return $input;
}

sub load_module {
	my $funcp = $::functions{"load_module"};
	return &$funcp(@_);
}

sub print_error {
	my $funcp = $::functions{"print_error"};
	return &$funcp(@_);
}

1;
